﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;



namespace k_thermo_pid
{
    public partial class Form1 : Form
    {
        string ComNumber;
        delegate void ShowGraph(Chart chart);

        public Form1()
        {
            InitializeComponent();

            // チャートの初期化設定
            initChart(chart);

            CDCcom comport = new CDCcom("VID_04D8&PID_000A");
            ComNumber = comport.getComNumber();

            if (ComNumber == null)
            {
                MessageBox.Show("CDC COM Port is not found!");
                Environment.Exit(0);
            }

            serialPort1.BaudRate = 57600;
            serialPort1.PortName = ComNumber;
            serialPort1.Open();


            serialPort1.ReadTimeout = 5000;// System.IO.Ports.SerialPort.InfiniteTimeout;
            serialPort1.DiscardInBuffer();

            label_sv.Text = temp;
        }

        private void initChart(Chart chart)
        {

            // チャート全体の背景色を設定
            chart.BackColor = Color.MidnightBlue;
            chart.ChartAreas[0].BackColor = Color.Transparent;

            // チャート表示エリア周囲の余白をカットする
            chart.ChartAreas[0].InnerPlotPosition.Auto = false;
            chart.ChartAreas[0].InnerPlotPosition.Width = 100; // 100%
            chart.ChartAreas[0].InnerPlotPosition.Height = 88;  // 80%(横軸のメモリラベル印字分の余裕を設ける)
            chart.ChartAreas[0].InnerPlotPosition.X = 8;
            chart.ChartAreas[0].InnerPlotPosition.Y = 0;


            // X,Y軸情報のセット関数を定義
            Action<Axis> setAxis = (axisInfo) =>
            {

                // 軸のメモリラベルのフォントサイズ上限値を制限
                axisInfo.LabelAutoFitMaxFontSize = 14;

                // 軸のメモリラベルの文字色をセット
                axisInfo.LabelStyle.ForeColor = Color.White;

                // 軸タイトルの文字フォントをセット
                axisInfo.TitleFont = new Font("ＭＳ 明朝", 12, FontStyle.Regular);

                // 軸タイトルの文字色をセット
                axisInfo.TitleForeColor = Color.White;

                // 軸の色をセット
                axisInfo.LineColor = Color.White;
                axisInfo.MajorGrid.Enabled = false;
                axisInfo.MajorGrid.LineColor = Color.White;// ColorTranslator.FromHtml("#008242");


            };

            // X,Y軸の表示方法を定義
            setAxis(chart.ChartAreas[0].AxisY);
            setAxis(chart.ChartAreas[0].AxisX);

            chart.ChartAreas[0].AxisY.Minimum = 20;    // 縦軸の最小値を20にする
            chart.ChartAreas[0].AxisX.Minimum = 0;    // 縦軸の最小値を0にする
            chart.ChartAreas[0].AxisY.Maximum = 60;    // 縦軸の最大値を60にする
            chart.ChartAreas[0].AxisX.Maximum = 60;    // 縦軸の最大値を60にする

            chart.ChartAreas[0].AxisX.LineWidth = 1;
            chart.ChartAreas[0].AxisY.LineWidth = 1;
            chart.ChartAreas[0].AxisX.Interval = 10;
            chart.ChartAreas[0].AxisY.Interval = 5;

            //           chart.AntiAliasing = AntiAliasingStyles.None;

            chart.ChartAreas[0].AxisX.Title = "時間　（ 分 ）　";  // X軸タイトル設定
            chart.ChartAreas[0].AxisY.Title = "温度　（ ℃ ）";  // Y軸タイトル設定

            chart.ChartAreas[0].AxisY.MajorGrid.Enabled = true;
            chart.ChartAreas[0].AxisY.MajorGrid.LineColor = Color.LightGreen;

            // 折れ線グラフとして表示
            chart.Series[0].ChartType = SeriesChartType.Line;

            // 線の色を指定
            chart.Series[0].Color = Color.White;// ColorTranslator.FromHtml("#00FF00");

            // 凡例を非表示,各値に数値を表示しない
            chart.Series[0].IsVisibleInLegend = false;
            chart.Series[0].IsValueShownAsLabel = false;

            chart.Series[0].Points.Clear();

            chart.Series[1].Points.Clear();
            chart.Series[1].Points.AddXY(60, 20);//グラフの上と左を閉じる
            chart.Series[1].Points.AddXY(60, 60);
            chart.Series[1].Points.AddXY(0, 60);
            chart.Series[1].BorderWidth = 1;//線の太さ

            chart.Series[1].ChartType = SeriesChartType.Line;
            chart.Series[1].Color = Color.White;
            chart.Series[1].IsValueShownAsLabel = false;
            chart.Series[1].IsVisibleInLegend = false;

            chart.Series[2].Color = Color.Yellow;
            chart.Series[2].IsValueShownAsLabel = false;
            chart.Series[2].IsVisibleInLegend = false;
        }

        int count;
        double num;
        byte[] dat = new byte[64];
        byte[] dat1 = new byte[64];
        bool f_process = false;
        bool f_measure = false;
        string pb = "";
        string ti = "";
        string td = "";
        string temp = "0";
        string lower = "20";
        string upper = "60";
        string max_voltage = "0";
        string text;


        private void multiMediaTimerComponent1_OnTimer(object sender)
        {
            if (f_process == false) this.Invoke(new ShowGraph(showChart), chart);

            //            if (f_process == false) showChart(chart);
        }

        // m: measure 温度測定
        // g: go 測定開始
        // t: temperature 温度設定            
        // r: pid read  データフォーマット：'p' +（pの値5桁）+'i' (iの値5桁) + 'd' (dの値5桁)
        // w: pid write  データフォーマット：'w'+ 'p' +（pの値5桁）+'i' (iの値5桁) + 'd' (dの値5桁)

        private void showChart(Chart chart)
        {
            f_process = true;

            dat[0] = (byte)'m';

            serialPort1.Write(dat, 0, 1);

            text = serialPort1.ReadLine();
            string str = Regex.Replace(text, @"[^0-9 | ^.]", "");
            num = double.Parse(str);
            label_temp.Text = str;

            if (count != 3599)//899)
                chart.Series[0].Points.AddXY(count * 0.05 / 3, num);
            else
                chart.Series[0].Points.AddXY(count * 0.05 / 3, Double.NaN);
            //値がない(非数)ことを示すDouble型フィールドで、これで帰線がなくなる。

            count++;

            chart.Series[2].Points.Clear();

            chart.Series[2].Points.AddXY(count * 0.05 / 3, lower);//移動バーの描画
            chart.Series[2].Points.AddXY(count * 0.05 / 3, upper);


            if (count == 3600) count = 0;// 900)    count = 0;

            if (chart.Series[0].Points.Count >= 3600)
                chart.Series[0].Points.RemoveAt(0);//以前のデータを消去する

            f_process = false;

        }




    //メニュー
    //測定開始
    private void Begin_Click(object sender, EventArgs e)
        {
            if (f_measure == false)
            {
                dat[0] = (byte)'g';
                serialPort1.Write(dat, 0, 1);
                text = serialPort1.ReadLine();

                multiMediaTimerComponent1.Enabled = true;

                begin.Text = "測定停止";
                begin.ForeColor = Color.Red;

                f_measure = true;
            }
            else
            {
                multiMediaTimerComponent1.Enabled = false;

                begin.Text = "測定開始";
                begin.ForeColor = Color.Black;

                f_measure = false;
            }
        }


        //温度設定
        private void Sv_Click(object sender, EventArgs e)
        {
            PID.Enabled = false;
            range.Enabled = false;
            sv.Enabled = false;
            maxvoltage.Enabled = false;

            Form4 form4  = new Form4();

            form4.temp.Text = temp;

            form4.RefreshEvent += delegate (object sender1, EventArgs e1)
            {
                dat = System.Text.Encoding.ASCII.GetBytes("t" + form4.temp.Text + "\0");
                serialPort1.Write(dat, 0, form4.temp.Text.Length + 2);
                text = serialPort1.ReadLine();
                temp = form4.temp.Text;
                label_sv.Text = temp;

                PID.Enabled = true;
                range.Enabled = true;
                sv.Enabled = true;
                maxvoltage.Enabled = true;
            };

            form4.CloseEvent += delegate (object sender1, EventArgs e1)
            {
                PID.Enabled = true;
                range.Enabled = true;
                sv.Enabled = true;
                maxvoltage.Enabled = true;
            };

            form4.Show();

        }


        //PID設定
        private void PID_Click(object sender, EventArgs e)
        {
            PID.Enabled = false;
            range.Enabled = false;
            sv.Enabled = false;
            maxvoltage.Enabled = false;

            Form2 form2 = new Form2();

            dat = System.Text.Encoding.ASCII.GetBytes("r" + "\0");

            serialPort1.Write(dat, 0, 2);
            text = serialPort1.ReadLine();


            pb=text.Substring(1, 5);
            pb = pb.Replace(" ", "");   //空白を除く
            form2.PBox.Text = pb;

            ti = text.Substring(7, 5);   //空白を除く
            ti = ti.Replace(" ", "");
            form2.IBox.Text = ti;

            td = text.Substring(13, 5);   //空白を除く
            td = td.Replace(" ", "");
            form2.DBox.Text = td;

            form2.RefreshEvent += delegate (object sender1, EventArgs e1)
            {

                dat = System.Text.Encoding.ASCII.GetBytes
                ("w"+"p" + String.Format("{0,5}", form2.PBox.Text)
                 +"i" + String.Format("{0,5}", form2.IBox.Text) 
                 + "d" + String.Format("{0,5}", form2.DBox.Text)+"\0");

                serialPort1.Write(dat, 0, 20);
                text = serialPort1.ReadLine();

                PID.Enabled = true;
                range.Enabled = true;
                sv.Enabled = true;
                maxvoltage.Enabled = true;
            };

            form2.CloseEvent += delegate (object sender1, EventArgs e1)
            {
                PID.Enabled = true;
                range.Enabled = true;
                sv.Enabled = true;
                maxvoltage.Enabled = true;
            };

            form2.Show();
        }


        //最大出力電圧の設定
        private void Maxvoltage_Click(object sender, EventArgs e)
        {
            PID.Enabled = false;
            range.Enabled = false;
            sv.Enabled = false;
            maxvoltage.Enabled = false;

            Form5 form5 = new Form5();

            form5.voltage.Text = max_voltage;

            form5.RefreshEvent += delegate (object sender1, EventArgs e1)
            {
                max_voltage = form5.voltage.Text;

                dat = System.Text.Encoding.ASCII.GetBytes("v" + form5.voltage.Text + "\0");
                serialPort1.Write(dat, 0, form5.voltage.Text.Length + 2);
                text = serialPort1.ReadLine();

                PID.Enabled = true;
                range.Enabled = true;
                sv.Enabled = true;
                maxvoltage.Enabled = true;
            };

            form5.CloseEvent += delegate (object sender1, EventArgs e1)
            {
                PID.Enabled = true;
                range.Enabled = true;
                sv.Enabled = true;
                maxvoltage.Enabled = true;
            };

            form5.Show();
        }


        //温度表示範囲の設定
        private void Range_Click(object sender, EventArgs e)
        {
            PID.Enabled = false;
            range.Enabled = false;
            sv.Enabled = false;
            maxvoltage.Enabled = false;

            Form3 form3 = new Form3();

            form3.upper.Text = upper;
            form3.lower.Text = lower;

            form3.RefreshEvent += delegate (object sender1, EventArgs e1)
            {
                upper = form3.upper.Text;
                lower = form3.lower.Text;

                int to_fr = int.Parse(upper) - int.Parse(lower);

                if (to_fr <= 10) chart.ChartAreas[0].AxisY.Interval = 1;
                else if (to_fr <= 20) chart.ChartAreas[0].AxisY.Interval = 2;
                else if (to_fr <= 50) chart.ChartAreas[0].AxisY.Interval = 5;
                else if (to_fr <= 100) chart.ChartAreas[0].AxisY.Interval = 10;
                else if (to_fr <= 200) chart.ChartAreas[0].AxisY.Interval = 20;
                else if (to_fr <= 500) chart.ChartAreas[0].AxisY.Interval = 50;
                else chart.ChartAreas[0].AxisY.Interval = 100;

                chart.ChartAreas[0].AxisY.Minimum = int.Parse(lower);    // 縦軸の最小値を20にする
                chart.ChartAreas[0].AxisY.Maximum = int.Parse(upper);    // 縦軸の最大値を60にする
                chart.Series[1].Points.Clear();
                chart.Series[1].Points.AddXY(60, int.Parse(lower));//グラフの上と左を閉じる
                chart.Series[1].Points.AddXY(60, int.Parse(upper));
                chart.Series[1].Points.AddXY(0, int.Parse(upper));
                chart.Series[1].BorderWidth = 1;//線の太さ

                chart.Series[2].Points.Clear();

                chart.Series[2].Points.AddXY(count * 0.05 / 3, lower);//移動バーの描画
                chart.Series[2].Points.AddXY(count * 0.05 / 3, upper);

                PID.Enabled = true;
                range.Enabled = true;
                sv.Enabled = true;
                maxvoltage.Enabled = true;
            };

            form3.CloseEvent += delegate (object sender1, EventArgs e1)
            {
                PID.Enabled = true;
                range.Enabled = true;
                sv.Enabled = true;
                maxvoltage.Enabled = true;
            };

            form3.Show();
            form3.upper.Focus();
        }

    }
}
